package de.cliffbreak.varo;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.bukkit.Difficulty;
import org.bukkit.GameRule;
import org.bukkit.NamespacedKey;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.configuration.file.YamlConfiguration;

import de.cliffbreak.varo.commands.VaroCommand;
import de.cliffbreak.varo.listeners.BannedItemListener;
import de.cliffbreak.varo.listeners.ChatListener;
import de.cliffbreak.varo.listeners.CreatureSpawnListener;
import de.cliffbreak.varo.listeners.PlayerInteractNPCListener;
import de.cliffbreak.varo.listeners.EntityRegainHealthListener;
import de.cliffbreak.varo.listeners.PlayerClientOptionsChangeListener;
import de.cliffbreak.varo.listeners.PlayerDeathListener;
import de.cliffbreak.varo.listeners.PlayerJoinQuitListener;
import de.cliffbreak.varo.listeners.PlayerPreLoginListener;
import de.cliffbreak.varo.managers.NPCManager;
import de.cliffbreak.varo.uitls.BanUtils;
import de.cliffbreak.varo.uitls.PlayerCache;

public class Varo extends JavaPlugin {

    public File configurationFile = new File("plugins/CliffbreakVaro", "config.yml");
    public FileConfiguration config = YamlConfiguration.loadConfiguration(this.configurationFile);
    public NPCManager npcManager;
    public PlayerCache playerCache;
    public BanUtils banUtils;

    @Override
    public void onEnable() {

        getLogger().info("CliffbreakVaro is starting...");

        this.npcManager = new NPCManager(this);
        this.playerCache = new PlayerCache(this);
        this.banUtils = new BanUtils(this);

        // this.config.addDefault("Varo.Start", "TODO: StartDate");
        this.config.addDefault("Varo.Debug", false);
        this.config.addDefault("Varo.Bans", new ArrayList<String>());
        this.config.options().copyDefaults(true);
        this.saveConfiguration();

        getServer().getPluginManager().registerEvents(new PlayerPreLoginListener(this), this);
        getServer().getPluginManager().registerEvents(new PlayerJoinQuitListener(this), this);
        getServer().getPluginManager().registerEvents(new PlayerClientOptionsChangeListener(this), this);
        getServer().getPluginManager().registerEvents(new ChatListener(), this);
        getServer().getPluginManager().registerEvents(new EntityRegainHealthListener(), this);
        getServer().getPluginManager().registerEvents(new PlayerDeathListener(this), this);
        getServer().getPluginManager().registerEvents(new BannedItemListener(), this);
        getServer().getPluginManager().registerEvents(new CreatureSpawnListener(this), this);
        getServer().getPluginManager().registerEvents(new PlayerInteractNPCListener(this), this);

        getCommand("varo").setExecutor(new VaroCommand(this));

        for (World world : getServer().getWorlds()) {
            if (world.getEnvironment() == Environment.NORMAL) {
                // world.setDifficulty(Difficulty.HARD);
                world.setDifficulty(Difficulty.PEACEFUL); // TODO: REMOVE AFTER DEBUG!!
                world.setGameRule(GameRule.ANNOUNCE_ADVANCEMENTS, false);
                world.setGameRule(GameRule.REDUCED_DEBUG_INFO, true);
            }
        }
        getServer().removeRecipe(NamespacedKey.minecraft("fishing_rod"));
    }

    @Override
    public void onDisable() {
        getLogger().info("Stopping CliffbreakVaro!");
        this.playerCache.shutdown();
    }

    public void saveConfiguration() {
        try {
            getLogger().info("Saving Config...");
            this.config.save(this.configurationFile);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
